@@ -18,7 +18,7 @@ ssl._create_default_https_context = ssl._create_unverified_context
 from termcolor import cprint
 from inspect import currentframe
 
-__version__ = '0.1.33'
+__version__ = '0.1.34'
 
 __info__ = {
     '2022/01/04: 0.1.18 加入 __info__。',
@@ -35,6 +35,7 @@ __info__ = {
     '2022/06/29: 0.1.30 加入需要的module',
     '2022/07/02: 0.1.32 加入 DefaultOrderedDict'
     '2022/07/13: 0.1.33 加入 binance_copy_bot'
+    '2022/07/14: 0.1.34 加入 price_ticks_offset_dec() 要用 Decimal 避免float error'
 }
 
 _MYPYLIB_ROOT = os.path.abspath(os.path.dirname(__file__))
@@ -395,6 +396,106 @@ class timeIt:
         print(f'It took {(self.end_time - self.start_time).seconds} seconds to finish {self.prompt}.')
 
 
+########################
+# Decimal
+########################
+from decimal import Decimal
+
+
+def get_current_price_tick_dec(price: Decimal, down=False):
+    # print(f'get_current_price_tick: {price}')
+    if down:
+        if price <= Decimal('10'):
+            return Decimal('0.01')
+        if 10 < price <= Decimal('50'):
+            return Decimal('0.05')
+        if 50 < price <= Decimal('100'):
+            return Decimal('0.1')
+        if 100 < price <= Decimal('500'):
+            return Decimal('0.5')
+        if 500 < price <= Decimal('1000'):
+            return Decimal('1')
+        if price > Decimal('1000'):
+            return Decimal('5')
+    else:
+        if price < Decimal('10'):
+            return Decimal('0.01')
+        if 10 <= price < Decimal('50'):
+            return Decimal('0.05')
+        if 50 <= price < Decimal('100'):
+            return Decimal('0.1')
+        if 100 <= price < Decimal('500'):
+            return Decimal('0.5')
+        if 500 <= price < Decimal('1000'):
+            return Decimal('1')
+        if price >= Decimal('1000'):
+            return Decimal('5')
+
+
+# price1 should be .LE. to price2
+def get_ticks_between_dec(_price1: Decimal, _price2: Decimal):
+    if _price2 < _price1:
+        price1 = _price2
+        price2 = _price1
+    else:
+        price1 = _price1
+        price2 = _price2
+    # print(price1, price2)
+    ticks = 0
+    while True:
+        price1 += get_current_price_tick_dec(price1)
+        # print(f'{ticks}: {price1} {price2}')
+        if price1 > price2:
+            break
+        ticks += 1
+    return ticks
+
+
+# TODO: Use decimal
+def price_ticks_offset_dec(price, ticks):
+    current_tick = get_current_price_tick(price)
+    price = round(price - (price + 0.001) % current_tick, 2)
+    # print(f'normalized price: {price}')
+    if ticks == 0:
+        return price
+    step = 1 if ticks > 0 else -1
+    for i in range(0, ticks, step):
+        current_tick = get_current_price_tick(price, down=True if step == -1 else False)
+        # print(i, price, current_tick)
+        price += current_tick * step
+    return round(price, 3)
+
+
+def get_limit_up_and_down_price_dec(price):
+    limit_up = price * 1.1
+    tick = get_current_price_tick(limit_up)
+    # By Carey
+    # df['漲停價'] = round(df['漲停價'] - ((df['漲停價']+0.001) % df['tick_up']),2)
+    limit_up = round(limit_up - (limit_up + 0.001) % tick, 2)
+
+    limit_down = price * 0.9
+    tick = get_current_price_tick(limit_down)
+    limit_down = math.ceil(limit_down / tick)
+    limit_down = limit_down * tick
+
+    return round(limit_up, 3), round(limit_down, 3)
+
+
+def price_stop_profit_and_lose_dec(price_enter, percentage_stop_profit, percentage_stop_lose, bool_call_or_put, tax=0.0015, fee=0.001425):
+    if bool_call_or_put:
+        price_stop_profit = price_ticks_offset(price_enter * (1 + percentage_stop_profit + fee * 2 + tax), 0)
+        price_stop_lose = price_ticks_offset(price_enter * (1 + percentage_stop_lose + fee * 2 + tax), 0)
+    else:
+        price_stop_profit = price_ticks_offset(price_enter * (1 - percentage_stop_profit - fee * 2 - tax), 0)
+        price_stop_lose = price_ticks_offset(price_enter * (1 - percentage_stop_lose - fee * 2 - tax), 0)
+
+    return price_stop_profit, price_stop_lose
+
+
+########################
+# Obselete
+########################
+
 def get_current_price_tick(price, down=False):
     # print(f'get_current_price_tick: {price}')
     if down:
@@ -747,7 +848,6 @@ if __name__ == '__main__':
         tline.send('測試')
         tline.stop()
 
-
     if True:
         tps = tplaysound()
         index = 0
